package com.dust.tripartite.netmc.core;

import com.dust.tripartite.netmc.core.annotation.AsyncBatch;
import com.dust.tripartite.netmc.core.annotation.Mapping;
import com.dust.tripartite.netmc.core.handler.AsyncBatchHandler;
import com.dust.tripartite.netmc.core.handler.Handler;
import com.dust.tripartite.netmc.core.handler.SimpleHandler;

import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Map;

/**
 * 消息处理映射
 * @author yezhihao
 * home https://gitee.com/yezhihao/jt808-server
 */
public abstract class AbstractHandlerMapping implements HandlerMapping {

    private final Map<Object, Handler> handlerMap = new HashMap<>(64);

    /**
     * 将Endpoint中被@Mapping标记的方法注册到映射表
     */
    protected synchronized void registerHandlers(Object bean) {
        Class<?> beanClass = bean.getClass();
        Method[] methods = beanClass.getDeclaredMethods();

        for (Method method : methods) {

            Mapping mapping = method.getAnnotation(Mapping.class);
            if (mapping != null) {

                String desc = mapping.desc();
                int[] types = mapping.types();

                AsyncBatch asyncBatch = method.getAnnotation(AsyncBatch.class);
                Handler handler;

                if (asyncBatch != null) {
                    handler = new AsyncBatchHandler(bean, method, desc, asyncBatch.poolSize(), asyncBatch.maxElements(), asyncBatch.maxWait());

                } else {
                    handler = new SimpleHandler(bean, method, desc);
                }

                for (int type : types) {
                    handlerMap.put(type, handler);
                }
            }
        }
    }

    /**
     * 根据消息类型获取Handler
     */
    public Handler getHandler(int messageId) {
        return handlerMap.get(messageId);
    }
}